
#include <stdlib.h>
#include <stdio.h>
#include <sys/time.h>
#include <malloc.h>


#include "multiply.h"

typedef unsigned long long  UINT64;

double TRIP_COUNT = (double)NUM * (double)NUM * (double)NUM;
int FLOP_PER_ITERATION = 2; // basic matrix multiplication


// routine to initialize an array with data
void init_arr(TYPE row, TYPE col, TYPE off, TYPE a[][NUM])
{
  int i,j;

  for (i=0; i< NUM;i++) {
    for (j=0; j<NUM;j++) {
      a[i][j] = row*i+col*j+off;
    }
  }
}

// routine to print out contents of small arrays
void print_arr(char *name, TYPE array[][NUM])
{
  int i,j;

  printf("\n%s\n", name);
  for (i=0;i<NUM;i++){
    for (j=0;j<NUM;j++) {
      printf("%g\t",array[i][j]);
    }
    printf("\n"); fflush(stdout);
  }
}

void multiply1(int msize, TYPE a[][NUM], TYPE b[][NUM], TYPE c[][NUM], TYPE t[][NUM])
{
	int i,j,k;

// Basic serial implementation
    for(i=0; i<msize; i++) {
        for(k=0; k<msize; k++) {
    	    for(j=0; j<msize; j++) {
				     c[i][j] = c[i][j] + a[i][k] * b[k][j];
			}
		}
	}
}

int main(int argc, char* argv[])
{

	double start=0.0, stop=0.0;
	struct timeval  before, after;
	double secs;
  double flops;
	double mflops;

	char *buf1,   *buf2, *buf3 , *buf4;
	char *addr1, *addr2, *addr3, *addr4;
	array *a, *b, *c, *t;
	int Offset_Addr1=128, Offset_Addr2=192, Offset_Addr3=0,Offset_Addr4=64;

// malloc arrays space
// Define ALIGNED in the preprocessor
// Also add '/Oa' for Windows and '-fno-alias' for Linux

	buf1 = (char *) malloc(NUM*NUM*(sizeof (double))+1024);
	printf("Addr of buf1 = %p\n",buf1); fflush(stdout);
	addr1 = buf1 + 256 - ((UINT64)buf1%256) + (UINT64)Offset_Addr1;
	printf("Offs of buf1 = %p\n",addr1); fflush(stdout);

	buf2 = (char *) malloc(NUM*NUM*(sizeof (double))+1024);
	printf("Addr of buf2 = %p\n",buf2); fflush(stdout);
	addr2 = buf2 + 256 - ((UINT64)buf2%256) + (UINT64)Offset_Addr2;
	printf("Offs of buf2 = %p\n",addr2); fflush(stdout);

	buf3 = (char *) malloc(NUM*NUM*(sizeof (double))+1024);
	printf("Addr of buf3 = %p\n",buf3); fflush(stdout);
	addr3 = buf3 + 256 - ((UINT64)buf3%256) + (UINT64)Offset_Addr3;
	printf("Offs of buf3 = %p\n",addr3); fflush(stdout);

	buf4 = (char *) malloc(NUM*NUM*(sizeof (double))+1024);
	printf("Addr of buf4 = %p\n",buf4); fflush(stdout);
	addr4 = buf4 + 256 - ((UINT64)buf4%256) + (UINT64)Offset_Addr4;
	printf("Offs of buf4 = %p\n",addr4); fflush(stdout);

	a = (array *) addr1;
	b = (array *) addr2;
	c = (array *) addr3;
	t = (array *) addr4;

// initialize the arrays with data
	init_arr(3,-2,1,a);
	init_arr(-2,1,3,b);


	gettimeofday(&before, NULL);

	multiply1(NUM, a, b, c, t);

	gettimeofday(&after, NULL);
  secs = (after.tv_sec - before.tv_sec) + (after.tv_usec - before.tv_usec)/1000000.0;


	flops = TRIP_COUNT * FLOP_PER_ITERATION;
	mflops = flops / 1000000.0f / secs;
	printf("Execution time = %2.3lf seconds\n",secs);	fflush(stdout);

  //printf("MFLOPS: %2.3f mflops\n", mflops);

  // print simple test case of data to be sure multiplication is correct
  if (NUM < 5) {
    print_arr("a", a); fflush(stdout);
    print_arr("b", b); fflush(stdout);
    print_arr("c", c); fflush(stdout);
  }

	free (buf1);
	free (buf2);
	free (buf3);
	free (buf4);

}
